/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/
#include <stdlib.h>
#include "ER-EPD0154-1.h"
#include "Debug.h"
#include "GUI_Paint.h"


void EPD1IN54::EPD_Reset(void)
{
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
    EPD_RST_0;
    Dev->Dev_Delay_ms(200);
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
}

/*******************************************************************************
function:
		Write register address and data
*******************************************************************************/
void EPD1IN54::EPD_SendCommand(UBYTE Reg)
{
    EPD_DC_0;
    EPD_CS_0;
    SPI_Write_Byte(Reg);
    EPD_CS_1;
}

void EPD1IN54::EPD_SendData(UBYTE Data)
{
    EPD_DC_1;
    EPD_CS_0;
    SPI_Write_Byte(Data);
    EPD_CS_1;
}

/**
 *  @brief: Wait until the busy_pin goes LOW
 */
void EPD1IN54::EPD_WaitUntilIdle(void)
{
    while(EPD_BUSY_RD == 0) {
        Dev->Dev_Delay_ms(100);
    }
    DEBUG("e-paper free busy...\n");
}



UBYTE EPD1IN54::EPD_Init(void)
{
    Dev->System_Init();

    EPD_Reset();

    EPD_SendCommand(BOOSTER_SOFT_START);
    EPD_SendData(0x17);
    EPD_SendData(0x17);
    EPD_SendData(0x17);

    EPD_SendCommand(POWER_SETTING);
    EPD_SendData(0x03);
    EPD_SendData(0x00);
    EPD_SendData(0x2B);
    EPD_SendData(0x2B);
    EPD_SendData(0x09);
   
    EPD_SendCommand(POWER_ON);
    EPD_WaitUntilIdle();

    EPD_SendCommand(PANEL_SETTING);
    EPD_SendData(0xCF);

    EPD_SendCommand(TCON_RESOLUTION);
    EPD_SendData(0x98);
    EPD_SendData(0x00);
    EPD_SendData(0x98);
 
    EPD_SendCommand(VCOM_AND_DATA_INTERVAL_SETTING);
 //     EPD_SendData(0x8F);
    EPD_SendData(0xF7);
     
      	
    EPD_SendCommand(VCM_DC_SETTING_REGISTER);
    EPD_SendData(0x0A);
  
    return 0;
}

void EPD1IN54::EPD_Clear(void)
{
    UWORD Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    UWORD Height = EPD_HEIGHT;

    //send black data
    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0xFF);
        }
    }
    Dev->Dev_Delay_ms(2);

    //send red data
    EPD_SendCommand(DATA_START_TRANSMISSION_2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0xFF);
        }
    }
    Dev->Dev_Delay_ms(2);
}

void EPD1IN54::EPD_Display(void)
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width));
        }
    }
    Dev->Dev_Delay_ms(2);

    EPD_SendCommand(DATA_START_TRANSMISSION_2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width + Paint_Image.Image_Offset));
        }
    }
    Dev->Dev_Delay_ms(2);

    EPD_SendCommand(DISPLAY_REFRESH);
    EPD_WaitUntilIdle();
}

void EPD1IN54::EPD_Sleep(void)
{
    EPD_SendCommand(VCOM_AND_DATA_INTERVAL_SETTING);
    EPD_SendData(0x17);
    EPD_SendCommand(VCM_DC_SETTING_REGISTER);         //to solve Vcom drop
    EPD_SendData(0x00);
    EPD_SendCommand(POWER_SETTING);         //power setting
    EPD_SendData(0x02);        //gate switch to external
    EPD_SendData(0x00);
    EPD_SendData(0x00);
    EPD_SendData(0x00);
    EPD_WaitUntilIdle();
    EPD_SendCommand(POWER_OFF);         //power off
}

